
/*
 * Ez egy DDS mely hrom hullmformt kpes egyidben ellltani. Ezek kzl
 * egyik frekvencija lland: 1 kHz. A msodikat ngy rtk kzl lehet ki-
 * vlasztani. Ennek ismeretben a harmadikat a rendszer szmtja ki gy hogy
 * 600 Hz-el tbb legyen a msodiknl. A frekvenciaprok olyan rtkek hogy
 * nincs kztk tfeds.
 *
 * A frekvencik kiosztsa:
 *
 * sorszm	1. kimenet	2. kimenet	3. kimenet
 *
 *   0.		1,0 kHz		1,6 kHz		2,2 kHz
 *   1.		1,0 kHz		2,8 kHz		3,4 kHz
 *   2.		1,0 kHz		4,0 kHz		4,6 kHz
 *   3.		1,0 kHz		5,2 kHz		5,8 kHz
 *
 *
 * Ezek kzl az MCU kt lbval lehet vlasztani. A rendszer az indulsakor
 * beolvassa ezek llapott s egy tblzatbl kivlasztja a frekvencikhoz
 * tartoz fzis delta rtkeket. A DDS ciklus ezekkel fogja lptetni a fzis
 * akkumultorokat. A hullmtbla egy 256 bjt mret tmb. Egy peridus hosz-
 * szsg szinuszjel pillanatrtkei vannak benne. A jel ellltsa gy zajlik
 * hogy a DDS egy fzis akkumultor legfls nyolc bitjt eltolsi cmnek hasz-
 * nlja s ezzel olvassa ki a soron kvetkez mintt amit kir arra portra ame-
 * lyik az illet fzis akkumultorhoz tartozik. Ott egy D/A talakt fogadja
 * a nyolc bites adatot.
 * Ezen fell a rendszer llandan figyeli egy lb llapott. Ha itt flfut
 * lt kap akkor egy kimeneti lbon harminc msodperces impulzust llt el.
 * Ha az impulzus ideje alatt jabb indtjelet kap, azzal nem trdik. Teht
 * egy jra nem indthat monostabil idztt valst meg. Ezt a tovbbi fel-
 * adatot gy vgzi hogy nem zavarja vele a szinuszjelek ellltst.
 *
 *
 * A programot ksztette:
 *
 *		Molnr Kroly
 *
 *		proton7@freemail.hu
 *
 */

/*
    Ezzel az llandval lehet gy fordtani a programot hogy az MCU kls kvarc-
    oszcilltorrl mkdjn.
*/
#define EXTERNAL_OSC

#include <pic18fregs.h>

#include "timer.h"

#pragma stack 0x60 64

#ifdef EXTERNAL_OSC
__code char __at __CONFIG1H _conf1  = _IESO_OFF_1H & _FCMEN_OFF_1H & _PRICLKEN_ON_1H &
				      _PLLCFG_ON_1H & _FOSC_HSMP_1H;
#else
__code char __at __CONFIG1H _conf1  = _IESO_OFF_1H & _FCMEN_OFF_1H & _PRICLKEN_OFF_1H &
				      _PLLCFG_ON_1H & _FOSC_INTIO67_1H;
#endif

__code char __at __CONFIG2L _conf2  = _BORV_285_2L & _BOREN_ON_2L & _PWRTEN_ON_2L;
__code char __at __CONFIG2H _conf3  = _WDTPS_1_2H & _WDTEN_OFF_2H;
__code char __at __CONFIG3H _conf5  = _MCLRE_EXTMCLR_3H & _HFOFST_OFF_3H & _PBADEN_OFF_3H & _CCP2MX_PORTC1_3H;
__code char __at __CONFIG4L _conf6  = _DEBUG_OFF_4L & _XINST_OFF_4L & _LVP_OFF_4L & _STVREN_ON_4L;
__code char __at __CONFIG5L _conf8  = _CP0_OFF_5L & _CP1_OFF_5L;
__code char __at __CONFIG5H _conf9  = _CPD_OFF_5H & _CPB_OFF_5H;
__code char __at __CONFIG6L _conf10 = _WRT0_ON_6L & _WRT1_ON_6L;
__code char __at __CONFIG6H _conf11 = _WRTD_OFF_6H & _WRTC_ON_6H & _WRTB_ON_6H;
__code char __at __CONFIG7L _conf12 = _EBTR0_OFF_7L & _EBTR1_OFF_7L;
__code char __at __CONFIG7H _conf13 = _EBTRB_OFF_7H;

typedef union
  {
  unsigned char all;

  struct
    {
    unsigned f0:1;
    unsigned f1:1;
    unsigned f2:1;
    unsigned f3:1;
    unsigned f4:1;
    unsigned f5:1;
    unsigned f6:1;
    unsigned f7:1;
    };
  } flags_t;

	/*
	    Egy DDS rendszer alapszmtsai:

		  M * fc
	    fo = --------
		   2^N

		 fo * 2^N
	    M = ----------
		    fc

	    fo: A kimen frekvencia.
	    fc: A DDS rajelfrekvencija.
	    M: A delta fzis regiszter rtke.
	    N: A rendszer bitszma (ltalban 24 s 32 bit kztt).

	    A frekvencia felbonts: fc / 2^N
	*/

	/*
	    Ha szksg van r akkor az albbi kt llandt egytt, szinkronban kell
	    mdostani. Mindkett a TMR0 idzt elosztjnak llst adja meg, de
	    klnbzik a flhasznlsi helyk.

	A TMR0_PRESCALER rtkei a kvetkezk lehetnek:

	      1
	      2
	      4
	      8
	     16
	     32
	     64
	    128
	    256

	A T0_PSCALER rtkei a kvetkezk lehetnek:

	    T0_PS_1_1
	    T0_PS_1_2
	    T0_PS_1_4
	    T0_PS_1_8
	    T0_PS_1_16
	    T0_PS_1_32
	    T0_PS_1_64
	    T0_PS_1_128
	    T0_PS_1_256
	*/
#define	TMR0_PRESCALER		256
#define	T0_PSCALER		T0_PS_1_256
#define	TMR0_1s			_Timer_Period_to_Value(65535, 1000000, TMR0_PRESCALER)

	/*
	    A TMR1 elosztsnak belltsra ugyanazok a szablyok rvnyesek
	    mint a TMR0 esetben.

	A TMR1_PRESCALER rtkei a kvetkezk lehetnek:

	    1
	    2
	    4
	    8

	A T1_PSCALER rtkei a kvetkezk lehetnek:

	    T1_PS_1_1
	    T1_PS_1_2
	    T1_PS_1_4
	    T1_PS_1_8
	*/
#define	TMR1_PRESCALER		1
#define	T1_PSCALER		T1_PS_1_1

	// A hullmalakot elllt megszaktsi ciklus 250.000 Hz-en jr.
#define DDS_MACHINE_CYCLES	64

	// A kplet az eredmnyt a legkzelebbi egsz fel kerekti.
#define	DDS_FREQ		(((MCU_OSC_FREQ / 2 / DDS_MACHINE_CYCLES) + 1) / 2)	// [Hz]

	// A genertor 250 kHz-en mkdik.
#define	TMR1_250kHz		_Timer_Freq_to_Value(65535, DDS_FREQ, TMR1_PRESCALER)

	// A frekvencibl kiszmolja a fzis akkumultor lpskzt.
#define	MakeDelta(freq)		(unsigned long)(4294967296.0 / (float)DDS_FREQ * (float)(freq) + 0.5)

	// Az els szinusz kimenet.
#define GUN_OUT			LATB
#define aGUN_OUT		__CONCAT(_, GUN_OUT)
#define GUN_DIR			TRISB

	// A msodik szinusz kimenet.
#define RING_OUT		LATC
#define aRING_OUT		__CONCAT(_, RING_OUT)
#define RING_DIR		TRISC

	// A harmadik szinusz kimenet.
#define REAR_LED_OUT		LATD
#define aREAR_LED_OUT		__CONCAT(_, REAR_LED_OUT)
#define REAR_LED_DIR		TRISD

	// Ezen a bemeneten a 'H' szint bekapcsolja a teszt zemmdot.
#define	TEST_INPUT		PORTEbits.RE0

	// Ez a bemenet indtja a tzelst.
#define	FIRE_INPUT		PORTEbits.RE1

	// A tilt jelet rzkel bemenet. Ide jn a tallat jelzse.
#define HIT_INPUT		PORTAbits.RA2

	// A tiltst vgz kimenet.
#define INHIBIT_OUT		LATAbits.LATA3
#define INHIBIT_OUT_DIR		TRISAbits.TRISA3

	// A hang MCU-t vezrl kt kimeneti lb.
#define SOUND_BIT0		LATAbits.LATA4
#define SOUND_DIR0		TRISAbits.TRISA4
#define SOUND_BIT1		LATAbits.LATA5
#define SOUND_DIR1		TRISAbits.TRISA5

#define	Silence()		SOUND_BIT0 = 0; SOUND_BIT1 = 0
#define	Sound_fire()		SOUND_BIT0 = 1; SOUND_BIT1 = 0
#define	Sound_hit()		SOUND_BIT0 = 0; SOUND_BIT1 = 1
#define	Sound_hit_test()	SOUND_BIT0 = 1; SOUND_BIT1 = 1

#define	WAVE_TABLE_SIZE		256

	/*
	    Ez a tmb a szinusz hullmforma egy teljes peridust tartalmazza
	    nyolc bites flbontssal. A hullmalak 0 foknl kezddik. A tmb
	    olyan cmre kerlt ami maradk nlkl oszthat 256-al. Ez azrt
	    van gy mert a mintkat kiolvas ciklus ezzel jelentsen
	    egyszerbb s sokkal gyorsabb lehet.
	*/

	// A szinuszjel.

static __code unsigned char __at(0x400) sin_table[WAVE_TABLE_SIZE] =
  {
  128,  131,  134,  137,
  140,  143,  146,  149,
  152,  155,  158,  162,
  165,  167,  170,  173,
  176,  179,  182,  185,
  188,  190,  193,  196,
  198,  201,  203,  206,
  208,  211,  213,  215,
  218,  220,  222,  224,
  226,  228,  230,  232,
  234,  235,  237,  238,
  240,  241,  243,  244,
  245,  246,  248,  249,
  250,  250,  251,  252,
  253,  253,  254,  254,
  254,  255,  255,  255,
  255,  255,  255,  255,
  254,  254,  254,  253,
  253,  252,  251,  250,
  250,  249,  248,  246,
  245,  244,  243,  241,
  240,  238,  237,  235,
  234,  232,  230,  228,
  226,  224,  222,  220,
  218,  215,  213,  211,
  208,  206,  203,  201,
  198,  196,  193,  190,
  188,  185,  182,  179,
  176,  173,  170,  167,
  165,  162,  158,  155,
  152,  149,  146,  143,
  140,  137,  134,  131,
  128,  124,  121,  118,
  115,  112,  109,  106,
  103,  100,   97,   93,
   90,   88,   85,   82,
   79,   76,   73,   70,
   67,   65,   62,   59,
   57,   54,   52,   49,
   47,   44,   42,   40,
   37,   35,   33,   31,
   29,   27,   25,   23,
   21,   20,   18,   17,
   15,   14,   12,   11,
   10,    9,    7,    6,
    5,    5,    4,    3,
    2,    2,    1,    1,
    1,    0,    0,    0,
    0,    0,    0,    0,
    1,    1,    1,    2,
    2,    3,    4,    5,
    5,    6,    7,    9,
   10,   11,   12,   14,
   15,   17,   18,   20,
   21,   23,   25,   27,
   29,   31,   33,   35,
   37,   40,   42,   44,
   47,   49,   52,   54,
   57,   59,   62,   65,
   67,   70,   73,   76,
   79,   82,   85,   88,
   90,   93,   97,  100,
  103,  106,  109,  112,
  115,  118,  121,  124
  };

#define FIRST_FREQ	1000		// [Hz]
#define FREQ_STEP	600		// [Hz]

	/*
	    Ez a tblzat a fzis delta rtkeket tartalmazza minden frekvencihoz.
	    Jelen belltsokkal a tblzat 1 kHz-el indul s 5,8 kHz-el r vget.
	*/

static __code unsigned long fase_deltas[] =
  {
  MakeDelta(FIRST_FREQ +  0 * FREQ_STEP),	// 1,0 kHz
  MakeDelta(FIRST_FREQ +  1 * FREQ_STEP),	// 1,6 kHz
  MakeDelta(FIRST_FREQ +  2 * FREQ_STEP),	// 2,2 kHz
  MakeDelta(FIRST_FREQ +  3 * FREQ_STEP),	// 2,8 kHz
  MakeDelta(FIRST_FREQ +  4 * FREQ_STEP),	// 3,4 kHz
  MakeDelta(FIRST_FREQ +  5 * FREQ_STEP),	// 4,0 kHz
  MakeDelta(FIRST_FREQ +  6 * FREQ_STEP),	// 4,6 kHz
  MakeDelta(FIRST_FREQ +  7 * FREQ_STEP),	// 5,2 kHz
  MakeDelta(FIRST_FREQ +  8 * FREQ_STEP)	// 5,8 kHz
  };

	// ltalnos idzt, ami msodpercenknt lp.
static unsigned char second_counter;

	// A tzelseket szmolja.
static unsigned char fire_counter;

	/*
	    Az albbi vltozk arra a RAM terletre kerlnek amit bankvlts
	    nlkl el lehet rni. Erre a minl gyorsabb kd miatt van szksg.
	    Ha a kd msik MCU-ra kerl akkor ezeket a cmeket ellenrizni
	    kell mert ez a RAM terlet nha nem a 0x7F cmen r vget hanem
	    hamarabb fejezdik be a 0x5F cmen.
	*/

#pragma udata access gun_fase_delta gun_fase_accumulator

	// A fzis-lpskz az els frekvencihoz.
static unsigned long __at(0x48) gun_fase_delta;
	// Az els fzis akkumultor.
static unsigned long __at(0x4C) gun_fase_accumulator;

#pragma udata access ring_fase_delta ring_fase_accumulator

	// A fzis-lpskz a msodik frekvencihoz.
static unsigned long __at(0x50) ring_fase_delta;
	// A msodik fzis akkumultor.
static unsigned long __at(0x54) ring_fase_accumulator;

#pragma udata access rear_LED_fase_delta rear_LED_fase_accumulator

	// A fzis-lpskz a harmadik frekvencihoz.
static unsigned long __at(0x58) rear_LED_fase_delta;
	// A harmadik fzis akkumultor.
static unsigned long __at(0x5C) rear_LED_fase_accumulator;

	// Ez a vltoz llapotjelzket tartalmaz.
static flags_t flags;

#define HIT_NOW		flags.f0	// A tallatjelzs jelen llapota.
#define HIT_PREV	flags.f1	// A tallatjelzs elz llapota.

#define FIRE_NOW	flags.f2	// A tzels indt bemenet jelen llapota.
#define FIRE_PREV	flags.f3	// A tzels indt bemenet elz llapota.

#define FIRE_ACTIVE	flags.f4	// A tzels idszakt jelzi. (30 msodperc)
#define FIRE_PAUSED	flags.f5	// A tzelsek kztti sznetet jelzi. (30 msodperc)

	// Ennyi alkalommal lehet tzelni.
#define	MAX_FIRE_COUNT		5

	// A tzels idtartama.
#define FIRE_TIME		30	// [msodperc]

	// A tzels utni sznet idtartama.
#define FIRE_PAUSE_TIME		30	// [msodperc]

	// A tallat utn a robot ennyi idn t cselekvskptelen.
#define	INHIBIT_TIME		60	// [msodperc]

//##############################################################################

	/*
	    A kiszolgls futsi ideje 58 vagy 59 ciklus. A jelen rajel
	    frekvencival 64 gpi ciklusonknt kvetkezik be jabb megszakts.
	*/

static void high_level_int(void) __interrupt 1 __naked
  {
  __asm
	goto	_high_int
  __endasm;
  }

static void high_int(void) __naked
  {
	/*
	    A megszakts egy gpi ciklus bizonytalansggal kvetkezik be. Ez ltalban
	    nem szokott gondot okozni, de itt frekvencit kell ellltani amit illene
	    preczen vgezni. Ez a kis makr megsznteti ezt a bizonytalansgot s vele
	    a fzishibt. A trkk az hogy az idztk nulladik bitje mutatja az eltrst
	    amit egy szksg esetn beiktatott kis ksleltets gygytani tud.
	*/
  Dejitter_TMR1();

	/*
	    Ha periodikus jelet kell ellltani egy idztvel akkor a kvetkez
	    peridusra ktfle mdon lehet elkszteni:

		1.)	jra be kell tlteni a kezdrtket.
		2.)	A jelenlegi - ismeretlen - llshoz hozz kell adni a kezdrtket.

	    Az els esettel az a baj hogy id telik el a tlcsorduls s az rdemi munka kztt.
	    A megszakts kiszolgls elindulsa is id s ezutn a fordt mg regisztereket
	    is ment (hacsak meg nem gtolom ebben). Radsul mg elfordulhat hogy azt a mveletet
	    is elre kell venni ami miatt az idzt dolgozik. A lnyeg hogy az idzts hosszabb
	    lesz a kelletnl, radsul mg ingadozhat is az idtartama. A hiba kivdshez
	    pontosan kell tudni hogy hny gpi ciklus telik el az idzt tlcsordulsa s az j
	    kezdrtk betltse kztt. Ezt hozz kell adni a kezdrtkhez - elre szmllrl
	    van sz - s ezt tlteni az idztbe. A kss mrtkt gy lehet megllaptani hogy
	    prbafordts utn bele kell nzi a kpzdtt assembly kdba. Ez gondot okozhat annak
	    a programoznak aki nem rti ezt a nyelvet. A msodik mdszer thidalja ezt a problmt,
	    mert az idztben ott van az az idtartam ami eltelt a tlcsorduls ta. Ehhez csak
	    hozz kell adni a kezdrtket s a peridusid lland lesz. Az sszeads elvgzsekor
	    arra kell vigyzni hogy a maga a mvelet is idbe kerl amit le kell vonni a kezdrtkbl.
	    (Teht mgiscsak kell nmi assembly ismeret. :-/ )
	*/
  Add16Timer1Lit(TMR1_250kHz);

	// A fntiek a megszaktstl kezdve 13/14 gpi ciklust emsztenek fl.

	// Ez maga a DDS.
  __asm
	movff	(_gun_fase_accumulator + 3), _TBLPTRL	//
	tblrd*						//
	movff	_TABLAT, aGUN_OUT			//
	movf	_gun_fase_delta, w				//
	addwf	_gun_fase_accumulator, f			//
	movf	(_gun_fase_delta + 1), w			// Ez 14 gpi ciklus.
	addwfc	(_gun_fase_accumulator + 1), f		//
	movf	(_gun_fase_delta + 2), w			//
	addwfc	(_gun_fase_accumulator + 2), f		//
	movf	(_gun_fase_delta + 3), w			//
	addwfc	(_gun_fase_accumulator + 3), f		//

	movff	(_ring_fase_accumulator + 3), _TBLPTRL	//
	tblrd*						//
	movff	_TABLAT, aRING_OUT			//
	movf	_ring_fase_delta, w				//
	addwf	_ring_fase_accumulator, f			//
	movf	(_ring_fase_delta + 1), w			// Ez 14 gpi ciklus.
	addwfc	(_ring_fase_accumulator + 1), f		//
	movf	(_ring_fase_delta + 2), w			//
	addwfc	(_ring_fase_accumulator + 2), f		//
	movf	(_ring_fase_delta + 3), w			//
	addwfc	(_ring_fase_accumulator + 3), f		//

	movff	(_rear_LED_fase_accumulator + 3), _TBLPTRL	//
	tblrd*						//
	movff	_TABLAT, aREAR_LED_OUT			//
	movf	_rear_LED_fase_delta, w				//
	addwf	_rear_LED_fase_accumulator, f			//
	movf	(_rear_LED_fase_delta + 1), w			// Ez 14 gpi ciklus.
	addwfc	(_rear_LED_fase_accumulator + 1), f		//
	movf	(_rear_LED_fase_delta + 2), w			//
	addwfc	(_rear_LED_fase_accumulator + 2), f		//
	movf	(_rear_LED_fase_delta + 3), w			//
	addwfc	(_rear_LED_fase_accumulator + 3), f		//
  __endasm;

  AckIntTimer1();

  __asm
	retfie	1
  __endasm;
  }

//------------------------------------------------------------------------------

	/*
	    A TMR0 idzt egy msodpercenknt kr megszakts kiszolglst.
	*/

static void low_level_int(void) __interrupt 2
  {
  if (isIntTimer0())
    {
    Dejitter_TMR0();
    Add16Timer0Lit(TMR0_1s);

    if (! second_counter || ! --second_counter)
      {
      if (FIRE_ACTIVE)
	{
	GUN_DIR = 0xFF;		// Befejezi a tzelst.
	FIRE_ACTIVE = 0;
	Silence();		// A tzels hangja elhal.
	FIRE_PAUSED = 1;	// Megkezdi a sznetet.
	second_counter = FIRE_PAUSE_TIME;
	}
      else if (FIRE_PAUSED)
	{
	FIRE_PAUSED = 0;	// Vge a sznetnek.
	StopTimer0();
	}
      else
	{
	// Letelt az idzts, vge a mozdulatlan llapotnak.
	INHIBIT_OUT = 0;
	REAR_LED_DIR = 0;	// A hts LED zemel.
	Silence();
	StopTimer0();
	}
      } // if (! second_counter || ! --second_counter)

    AckIntTimer0();
    } // if (isIntTimer0())
  }

//------------------------------------------------------------------------------

	/*
	    Ez egy egyszer DDS mdszerrel mkd hullmforma genertor.
	    A fzis akkumultor legfls nyolc bitje egy hullmtbla cmzst
	    vgzi. A kiolvasott minta maga a kimenjel. Ez a B porton jut ki
	    az MCU-bl. Itt egy 2R/R ltrahlzat alaktja t analg jell.

	    Ezt pedig egy hrom fokozat alultereszt szr smtja:

	          R           R           R

	       +-----+     +-----+     +-----+
	    o--+     +--*--+     +--*--+     +--*--o
	       +-----+  |  +-----+  |  +-----+  |
	                |           |           |
	             C ===       C ===       C ===
	                |           |           |
	                |           |           |
	               ---         ---         ---

		 R = 3,9 kOhm
		 C = 330 pF

	    A genertor fix frekvencikat llt el. Az els frekvencia az 1 kHz,
	    a tbbi pedig ezt kveti 600 Hz-es lpsekben.

	    A lbkioszts:

	    RE3:	MCU reset

	    RA0:1	A frekvencit kivlaszt bemenetek.

	    RA2:	A tallat jelzst fogad bemenet.

	    RA3:	Ha a robot tallatot kapott akkor ezen a kimeneten jelzi.
			    A tallat utn a robot pontosan egy percig kptelen
			    a tzelsre s a mozgsra.

	    RA4:5	A hang MCU-t vezrl kt kimenet.
			    00: csnd
			    01: a robot tzel
			    10: a robot tallatot kapott
			    11: folyamatos hang a belltshoz

	    RE0:	zemmd vlaszt bemenet.
			    0: verseny
			    1: bellts

	    RE1:	A "fegyver" vezrl bemenete.
			    H szintre bekapcsol a "fegyver" s 30 msodpercig
			    folyamatosan mkdik. Ha a robot kzben tallatot
			    kap akkor azonnal besznteti a tzelst.
			    A "fegyvert" legfljebb t alkalommal lehet bekapcsolni.
			    Kt lvs kztt legalbb 30 msodperc sznet van.

	    RB0:7	A "fegyver" kimenete.

	    RC0:7	A gyrt meghajt kimenet.

	    RD0:7	A hts LED-et meghajt kimenet.
	*/

void main(void)
{
  INTCON = 0;		// Tilt minden megszaktst.

#ifndef EXTERNAL_OSC
	/*
	    A bels RC oszcilltort vlasztja 16 MHz frekvencival.
	    A PLL ezt a ngyszeresre nveli ezrt 64 MHz lesz belle.
	    Ha az OSCCON-ban bels oszcilltort vlasztok ki akkor
	    a PLL kimarad a jel tjbl.
	*/
  OSCCON = 0b01110000;
  OSCCON2 = 0;
  OSCTUNEbits.PLLEN = 1;
	// Megvrja hogy lland legyen az oszcilltor jele.
  while (! OSCCONbits.IOFS);
#endif

  ANSELA = 0;		// Lekapcsol minden analg bemenetet.
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;

  CM1CON0 = 0;		// Lekapcsolja a kompartorokat.
  CM2CON0 = 0;

  CTMUCONH = 0;		// A kapacitsmrt is lekapcsolja.

	/*
	    Kikapcsol egy csom perifrit. (A TMR0-t nem lehet kikapcsolni,
	    radsul most szksg is van r.) Ez a lps cskkenti az ramfogyasztst.
	*/

  PMD0 = 0xFF;
  PMD0bits.TMR1MD = 0;	// Csak a TMR1 idztt engedlyezi.
  PMD1 = 0xFF;
  PMD2 = 0xFF;

  GUN_OUT = 0;
  GUN_DIR = 0xFF;	// A "fegyver" alaphelyzetben nem mkdik.
  RING_OUT = 0;
  RING_DIR = 0;		// A gyr mindig zemel.
  REAR_LED_OUT = 0;
  REAR_LED_DIR = 0;	// A hts LED mkdik.

  INHIBIT_OUT = 0;	// A robot mozoghat.
  INHIBIT_OUT_DIR = 0;

  Silence();		// Csndben kezdi.
  SOUND_DIR0 = 0;	//
  SOUND_DIR1 = 0;	//

  flags.all = 0;	// Trli az sszes llapotjelzt.

  RCONbits.IPEN = 1;		// Engedlyezi a ktszint megszakts kezelst.
  SetLowIntPriorityTimer0();	// A TMR0 idzt alacsony prioritst kap.
  SetHighIntPriorityTimer1();	// A TMR1 idzt magas prioritst kap.

	// A TMR0 adja az egy msodperces idalapot.
  ConfigTimer0(T0_16_BIT | T0_SOURCE_INT | T0_PSCALER);
  Write16Timer0Lit(TMR0_1s);
  EnIntTimer0();

	// A TMR1 adja a DDS idalapjt.
  ConfigTimer1(T1_16_BIT_RW | T1_PSCALER | T1_SOSC1EN_OFF | T1_SYNC_EXT_OFF | T1_SOURCE_FOSC_4);
  Write16Timer1Lit(TMR1_250kHz);
  _EnIntTimer1();

	// A tzels frekvencija.

  gun_fase_delta = fase_deltas[0];	// 1 kHz
  gun_fase_accumulator = 0;

	// Az autt kt frekvencia azonostja. Ezek egymstl 600 Hz-re vannak.

  FSR0L = PORTA & 0b00000011;		// Csak az RA0:1 bitek kellenek.
  FSR0L <<= 1;				// Egyszerre kt szomszdos frekvencit kell kivlasztani.
  ++FSR0L;				// Az 1 kHz foglalt rtk.

	// Az als frekvencia rtk.

  ring_fase_delta = fase_deltas[FSR0L];
  ring_fase_accumulator = 0;

	// A fls frekvencia rtk.

  rear_LED_fase_delta = fase_deltas[FSR0L + 1];
  rear_LED_fase_accumulator = 0;

  fire_counter = 0;			// Csak a Reset trli.

  bTBLPTR = sin_table;			// A hullmtbla mutat.

	/*
	    Mostantl kezdve semmi olyat nem szabad csinlni ami - terven fell -
	    megvltoztatja a TBLPTR tartalmt.
	*/

  INTCONbits.GIEL = 1;
  INTCONbits.GIEH = 1;

  StartTimer1();			// Indul a DDS.

	// A fprogram csak a bntssal foglalkozik.

  while (1)
    {
    if (TEST_INPUT)			// 'H' szint.
      {
	/*
	    A robot bellts zemmdban van. Ilyenkor nincsenek korltozsok.
	    Tzelni lehet amg jlesik s a tallatjelzs is addig tart amg
	    jel van a bemeneten.
	*/
      if (HIT_INPUT)
	{
	// Ha a tallatjelzs aktv akkor egy lland magassg hang szl.
	GUN_DIR = 0xFF;			// A "fegyver" pihen.
	REAR_LED_DIR = 0xFF;		// A hts LED kikapcsol.
	INHIBIT_OUT = 1;		// A robot lell.
	Sound_hit_test();		// Megszlal a hang.
	}
      else if (FIRE_INPUT)
	{
	// A "fegyver" folyamatosan kveti a vezrl bemenet llapott.
	GUN_DIR = 0;			// A "fegyver" tzel.
	REAR_LED_DIR = 0;		// A hts LED zemel.
	INHIBIT_OUT = 0;		// Szabad a mozgs.
	Sound_fire();			// Megszlal a tzelst jelz hang.
	}
      else
	{
	GUN_DIR = 0xFF;			// A "fegyver" pihen.
	REAR_LED_DIR = 0;		// A hts LED zemel.
	INHIBIT_OUT = 0;		// Szabad a mozgs.
	Silence();			// Csnd lesz.
	}
      } // if (TEST_INPUT)
    else if (! INHIBIT_OUT)
      {
	/*
	    A versenyen ebben az zemmdban van a robot. Ha kzben egy tallat
	    miatt mozdulatlansgra knyszerl akkor annak vgig semmit sem tud
	    tenni.
	*/
      HIT_NOW = 0;
      if (HIT_INPUT)
	{
	HIT_NOW = 1;			// Beolvassa a tallat bemenet llapott.
	}

      FIRE_NOW = 0;
      if (FIRE_INPUT)
	{
	FIRE_NOW = 1;			// Beolvassa a tzels bemenet llapott.
	}

      if (! HIT_PREV && HIT_NOW)
	{
	/*
	    Az elz llapot 'L' a mostani pedig 'H', teht az HIT_INPUT lbra
	    flfut l rkezett. Ez azt jelenti hogy a robot tallatot kapott.
	    Ezrt most 60 msodperces mozdulatlansg kvetkezik. Az indtjel
	    csak akkor juthat rvnyre ha a TMR0 idzt ll. Ez azt jelenti
	    hogy a mozdulatlansg ideje alat rkez tallatjelzst a rendszer
	    nem veszi figyelembe.

	    A TMR0 idzt minden msodpercben megszaktst kr. A second_counter
	    ilyenkor cskken egyel. Ha nullra r akkor vge a nyugalmi llapotnak.
	*/
	StopTimer0();			// A biztonsg kedvrt lelltja az idztt.

	GUN_DIR = 0xFF;			// Ha eddig tzelt akkor most abbahagyja.
	FIRE_ACTIVE = 0;
	FIRE_PAUSED = 0;

	REAR_LED_DIR = 0xFF;		// A hts LED kikapcsol.

	INHIBIT_OUT = 1;		// A robot lell.

	Sound_hit();			// Elkezdi a hangjelzst.

	second_counter = INHIBIT_TIME;	// A "bnasg" idtartama.
	Write16Timer0Lit(TMR0_1s);
	StartTimer0();			// Megkezdi a vrakozst.
	} // if (! HIT_PREV && HIT_NOW)
      else if (! FIRE_PREV && FIRE_NOW)
	{
	/*
	    Flfut l rkezett a bemenetre. Ez kezdemnyezi a tzelst, de ez
	    nem kvetkezik be flttel nlkl.
	*/
	if (fire_counter < MAX_FIRE_COUNT)
	  {
	// A tzelsek szma a megengedett alatt van.
	  if (! FIRE_ACTIVE && ! FIRE_PAUSED)
	    {
	// Jelenleg nincs tzels s a ktelez sznetnek is vge van.
	    GUN_DIR = 0;		// A "fegyver" mkdni kezd.
	    FIRE_ACTIVE = 1;

	    Sound_fire();
	    second_counter = FIRE_TIME;	// A tzels ideje 30 msodperc.
	    Write16Timer0Lit(TMR0_1s);
	    StartTimer0();		// Mrni kezdi az idt.
	    ++fire_counter;		// Szmolja a tzelst.
	    } // if (! FIRE_ACTIVE && ! FIRE_PAUSED)
	  } // if (fire_counter < MAX_FIRE_COUNT)
	} // else if (! FIRE_PREV && FIRE_NOW)

	// A kvetkez ciklusra elteszi a mostani llapotokat.
      HIT_PREV = 0;
      if (HIT_NOW)
	{
	HIT_PREV = 1;
	}

      FIRE_PREV = 0;
      if (FIRE_NOW)
	{
	FIRE_PREV = 1;
	}
      } // else if (! INHIBIT_OUT)
    } // while (1)
}
